      PROGRAM mono_identification

c     This code calculates the discriminant for mode identification from
c     line-profile variations according to the paper by Aerts, C., 1996,
c     A&A 314, 115, which is an improvement of the version first
c     published in Aerts C., De Pauw, M., Waelkens, C., 1992, A&A 266,
c     294. The original code written by C. Aerts was translated to this
c     version in Fortran by Peter De Cat & Katrien Uytterhoeven, both
c     PhD students of C. Aerts at the University of Leuven (Belgium).
c     Please refer to the papers by Aerts et al. (1992) and Aerts (1996)
c     whenever you publish results based on this code.

c     The inputfile is named discriminant.in; it requires the amplitudes
c     of the first, second, and third moment, together with the constant
c     of the second moment. Besides this, the codes needs a value for:
c
c     1) K: the ratio of the horizontal to vertical velocity amplitude
c     of the star, given by GM/f^2R^3 with f the pulsation frequency
c     (this formula can be derived from the boundary condition), 

c     2) the linear limb-darkening coefficient u 

c     3) the frequency f (in c/d) and the name of the file in which the
c     output is to be written.
c
c     If you want to calculate the discriminant for a multiperiodic
c     pulsation, you should use the code multi_discriminant.f for
c     each of the modes that you want to identify. The reason for using
c     a different code in this case is that you cannot make use of the
c     constant of the second moment, as explained in Mathias et
c     al. (1994, A&A 283, 813).

c     Good luck !  Conny Aerts, Peter De Cat, Katrien Uytterhoeven






C     variabelen

c     nmintable denotes the maximum number of (l,m)-couples
c     nmintable = 45 for l=8
      parameter (nmintable=45)

c     nminima denotes the number of sorted discriminant values for 
c     every (l,m)-couple
      parameter (nminima=50)

c     abscissas denotes the number of abscissas used in the method of
c     Gauss-Legendre for the calculation of the integrals
      parameter (abscissas=12)

c     nmax denotes the maximum size of the grid
      parameter (nmax=20000)

      character output*70

      integer   ni, nvp, nvo, nsig
      integer   li, lvp, lvo, lsig
      integer   l, m, ncouple, nmin
      integer   k, location, row, rownr 
      integer   j, spec, nmodes, imodes
      integer   lowl, upl, lowm, upm, count, percent, total

      real*8    table(nmax,5)
      real*8    mintable(nmintable,7)
      real*8    pi, minimum
      real*8            lowid, upid,   stepid
      real*8    incl,   lowi,  upi,    stepi
      real*8    vp,     lowvp,  upvp,  stepvp
      real*8    vomega, lowvo,  upvo,  stepvo
      real*8    sigma,  lowsig, upsig, stepsig
      real*8    beta, limbdark, Kvalue 
      real*8    gamma1, gamman
      real*8    absc(abscissas), w(abscissas)
      real*8    aa,  cc,  dd,  ee,  ff,  gg,  rst
      real*8    saa, scc, sdd, see, sff, sgg, srst
      real*8    freq, Sf
      real*8    discriminant

      character kind*13,  header(6)*70, ster*25

      common/integraal/absc,w
      common/limbdarkening/beta
      common/alfa/Kvalue
      common/error/Sf
  
      pi = 4*datan(1.0d0)

      call gauleg
 
      write(6,*) 
      write(6,*)

      open(1,file='discriminant.in')
      do j=1,4
         read(1,*,end=1000)
      enddo
      read(1,*) ster
      do j=1,2
         read(1,*,end=1000)
      enddo


c**** read part 1 of the inputfile : range of modes

      do j=1,2
         read(1,'(a70)',end=1000) header(j)
         write(6,'(a70)')         header(j)
      enddo
      read(1,*)

      read(1,*)  lowl,upl
      write(6,*) '---> range of l-values : ',lowl,upl
      read(1,*)  kind
      write(6,*) '---> kind of modes     : ',kind
      if ((kind.eq.'tesseral').and.(upl.lt.2)) then
         write(6,*) 
         stop 'No tesseral modes in specified l-range !!!'
      endif
      do j=1,5
         read(1,*,end=1000)
      enddo      
      read(1,*) spec
      if (kind.eq.'specific') then
         write(6,*) '---> specific m-value     : ',spec
      endif
c     calculation of the number of (l,m)-couples
      ncouple = 0
      do l = lowl, upl
         if (kind.eq.'all')                 ncouple = ncouple + (l+1)
         if (kind.eq.'axi-symmetric')       ncouple = ncouple + (1)
         if (kind.eq.'tesseral'.and.l.gt.1) ncouple = ncouple + (l-1)
         if (kind.eq.'sectoral')            ncouple = ncouple + (1)
         if (kind.eq.'specific')            ncouple = ncouple + (1)
      enddo
      write(6,*)
      write(6,*) '---> number of (l,m)-couples          : ',ncouple

      write(6,*)
      write(6,*)


c**** read part 2 of the inputfile : grid of parameters

      do j = 1, 2
         read(1,*,end=1000)
      enddo
      do j= 3, 4
         read(1,'(a70)',end=1000) header(j)
         write(6,'(a70)')         header(j)
      enddo
      read(1,*)

      read(1,*) lowid, upid, stepid
      ni   = int((upid-lowid)/stepid) + 1
      write(6,621) '---> inclination (degrees) : ',lowid, upid, stepid
       lowi = ( lowid*pi)/180
        upi = (  upid*pi)/180
      stepi = (stepid*pi)/180
      write(6,621) '                 (radians) : ',lowi,  upi,  stepi
      read(1,*) lowvp, upvp, stepvp
      nvp  = int((upvp-lowvp)/stepvp) + 1
      write(6,621) '---> vp (km/s)             : ',lowvp, upvp, stepvp
      read(1,*) lowvo, upvo, stepvo
      nvo  = int((upvo-lowvo)/stepvo) + 1
      write(6,621) '---> vomega (km/s)         : ',lowvo, upvo, stepvo
      read(1,*) lowsig,upsig,stepsig
      nsig = int((upsig-lowsig)/stepsig) + 1
      write(6,621) '---> sigma (km/s)          : ',lowsig,upsig,stepsig
 621  format(a29,3f10.5)

c     Determination of the total number of gridpoints for every (l,m)
c      total   = ((upi -lowi )/stepi +1)*((upvp -lowvp )/stepvp +1)*
c     &          ((upvo-lowvo)/stepvo+1)*((upsig-lowsig)/stepsig+1)
      total = ni*nvp*nvo*nsig
      percent = total*0.2
      if (percent.eq.0) percent = 1
      write(6,622) '---> number of gridpoints  : ',total,' => 20 % :',
     &                                             percent
      nmin = min(nminima,total)
      write(6,*) '---> number of minima for every (l,m) : ',nmin
 622  format(a29,i10,a10,i10)
      write(6,*)
      write(6,*)


c**** read part 3 of the inputfile : amplitude of moments

      do j=1,2
         read(1,*,end=1000)
      enddo
      do j=5,6
         read(1,'(a70)',end=1000) header(j)
         write(6,'(a70)')         header(j)
      enddo
      read(1,*,end=1000)

      read(1,*,end=1000)  limbdark
      beta = limbdark/(1-limbdark)
      write(6,630) '---> limbdarkening   : ',limbdark,' => beta : ',beta
 630  format(a23,f10.5,a11,f15.10)
      
      nmodes = 1
      

c**** main do-loop

      DO imodes = 1, nmodes, 1

         write(6,*) 
         write(6,*)

         read(1,*,end=1000)
         write(6,*)            '***********'
         write(6,'(a8,i2,a2)') ' * MODE ',imodes,' *'
         write(6,*)            '***********'
         write(6,*)

         read(1,*,end=1000)          output
         write(6,*)                  output
         read(1,*,end=1000)          freq
         write(6,634) '---> freq : ',freq,' c/d'
         read(1,*,end=1000)          Kvalue
         write(6,632) '---> K    : ',Kvalue 
         read(1,*,end=1000)          aa, saa
         write(6,633) '---> AA   : ',aa ,saa 
         read(1,*,end=1000)          cc, scc
         write(6,633) '---> CC   : ',cc ,scc 
         read(1,*,end=1000)          dd, sdd
         write(6,633) '---> DD   : ',dd ,sdd 
         read(1,*,end=1000)          ee, see
         write(6,633) '---> EE   : ',ee ,see
         read(1,*,end=1000)          ff, sff
         write(6,633) '---> FF   : ',ff ,sff 
         read(1,*,end=1000)          gg, sgg
         write(6,633) '---> GG   : ',gg ,sgg 
         read(1,*,end=1000)          rst,srst
         write(6,633) '---> RST  : ',rst,srst 
 632     format(a12,f12.5)
 633     format(a12,2f12.5)
 634     format(a12,f10.7,a4)

         count = 0


c******* write headerlines in outputfile

         open(2,file=output)

         write(2,*) output
         write(2,*) 
         write(2,*) ster
         write(2,*) 
         write(2,*) header(1)
         write(2,*) header(2)
         write(2,*) '---> range of l-values : ',lowl,upl
         write(2,*) '---> kind of modes     : ',kind
         write(2,*) '---> number of (l,m)-couples          : ',ncouple
         write(2,*) '---> number of minima for every (l,m) : ',nmin
         write(2,*) 
         write(2,*) header(3)
         write(2,*) header(4)
         write(2,621) '---> inclination (degrees) : ',
     &        lowid, upid, stepid
         write(2,621) '---> vp (km/s)             : ',
     &        lowvp, upvp, stepvp
         write(2,621) '---> vomega (km/s)         : ',
     &        lowvo, upvo, stepvo
         write(2,621) '---> sigma (km/s)          : ',
     &        lowsig,upsig,stepsig
         write(2,*) 
         write(2,*) header(5)
         write(2,*) header(6)
         write(2,630) '---> limbdarkening   : ',limbdark,
     &                ' => beta : ',beta
         write(2,634) '---> freq : ',freq,' c/d'
         write(2,632) '---> K    : ',Kvalue 
         write(2,633) '---> AA   : ',aa ,saa 
         write(2,633) '---> CC   : ',cc ,scc 
         write(2,633) '---> DD   : ',dd ,sdd 
         write(2,633) '---> EE   : ',ee ,see 
         write(2,633) '---> FF   : ',ff ,sff 
         write(2,633) '---> GG   : ',gg ,sgg 
         write(2,633) '---> RST  : ',rst,srst 
         write(2,*) 
         write(2,*) 

c******* loop for all the l-values

 123     DO l = lowl, upl, 1

c           Determine the range of m-values
            if (kind.eq.'all') then
               lowm = 0
               upm  = l
            endif
            if (kind.eq.'axi-symmetric') then
               lowm = 0
               upm  = 0
            endif
            if (kind.eq.'tesseral') then
               lowm = 1
               upm  = l-1
            endif
            if (kind.eq.'sectoral') then
               lowm = l
               upm  = l
            endif
            if (kind.eq.'specific') then
               lowm = spec
               upm  = spec
            endif

c********** loop for all the m-values

            DO m = lowm, upm, 1
               
               write(6,*) 
               write(6,*) '(l,m) = (',l,',',m,')'
               write(2,*) 
               write(2,202) 'pos','l','m','discr','vp','incl','vomega',
     &                      'sigma','dgamma_1','dgamma_n'
 202           format(a3,3x,a3,1x,a3,a11,2x,4a8,2x,2a10)
                  
               count = count + 1
               row      = 0
               
c************* loops for the grid of parameters

               vp     = lowvp  - stepvp
               incl   = lowi   - stepi
               vomega = lowvo  - stepvo
               sigma  = lowsig - stepsig

               DO lvp = 1, nvp
                  vp = vp + stepvp
                  DO li = 1, ni
                     incl = incl + stepi
                     DO lvo = 1, nvo
                        vomega = vomega + stepvo
                        DO lsig  = 1, nsig
                           sigma = sigma + stepsig

c************************* row = number of the iterations for this (l,m)

                           row = row + 1
                           if (mod(row,percent).eq.0) then
                              write(6,*) (row/percent)*20,' %'
                           endif

c************************* write on tape of the values
                           table(row,1) = discriminant(l,m,incl,vp,
     &                        vomega,sigma,aa,cc,dd,ee,ff,gg,
     &                        rst,saa,scc,sdd,see,sff,sgg,srst)
                           table(row,3) = (incl*180)/pi 
                           table(row,2) = vp
                           table(row,4) = vomega
                           table(row,5) = sigma

                        ENDDO
                        sigma = lowsig - stepsig
                     ENDDO
                     vomega = lowvo - stepvo
                  ENDDO
                  incl = lowi - stepi
               ENDDO
               vp = lowvp - stepvp


c************* search for the minima of the discriminant (sort by alternation)
c              the value of the lowest minimum will be written in mintable 
c              which contains the nmin values, associated with every pair 
c              of (l,m)

               minimum  = table(1,1)
               location = 1
               rownr    = 1
 14            if (location.le.nmin) then
                  do j = location+1, total, 1
                     if (table(j,1).le.minimum) then
                        minimum = table(j,1)
                        rownr   = j
                     endif
                  enddo
                  if (location.eq.1) then
                     gamma1 = table(rownr,1)
                     gamman = table(rownr,1)
                     write(6,202) 'pos','l','m','discr','vp','incl',
     &                            'vomega','sigma','dgamma_1',
     &                            'dgamma_n'
                     write(6,201) location, l, m, table(rownr,1), 
     &                            table(rownr,2), table(rownr,3), 
     &                            table(rownr,4), table(rownr,5),
     &                            table(rownr,1)-gamma1,
     &                            table(rownr,1)-gamman
                  endif
                  write(2,201) location, l, m, table(rownr,1), 
     &                         table(rownr,2), table(rownr,3), 
     &                         table(rownr,4), table(rownr,5),
     &                         table(rownr,1)-gamma1,
     &                         table(rownr,1)-gamman
 201              format(i3,3x,i3,1x,i3,f11.5,2x,4f8.3,2x,2f10.5)
                  if (location.eq.1) then
                     mintable(count,1) = l
                     mintable(count,2) = m
                  endif
                  gamman   = table(rownr,1)
                  do k = 1, 5, 1
                     if (location.eq.1) mintable(count,k+2) = 
     &                                  table(rownr,k)
                     table(rownr,k) = table(location,k)
                  enddo
                  location = location + 1
                  minimum  = table(location,1)
                  rownr    = location
                  goto 14
               endif

            ENDDO

         ENDDO
         
      ENDDO

      write(6,*)
      write(6,*)
      write(6,210) Sf
      write(6,*)
      write(6,202) 'pos','l','m','discr','vp','incl','vomega','sigma',
     &             'dgamma_1','dgamma_n'

      write(2,*)
      write(2,*)
      write(2,210) Sf
 210  format('Table with the final results (Sf = ',f10.5,')')
      write(2,*)
      write(2,202) 'pos','l','m','discr','vp','incl','vomega','sigma',
     &             'dgamma_1','dgamma_n'


      minimum  = mintable(1,3)
      location = 1
      rownr    = 1

 15   if (location.le.ncouple) then
         do j = location+1, ncouple, 1
            if (mintable(j,3).le.minimum) then
               minimum = mintable(j,3)
               rownr   = j
            endif
         enddo
         if (location.eq.1) then
            gamma1 = mintable(rownr,3)
            gamman = mintable(rownr,3)
         endif 
         l = mintable(rownr,1)
         m = mintable(rownr,2)
         write(2,201) location, l, m, mintable(rownr,3), 
     &                mintable(rownr,4), mintable(rownr,5), 
     &                mintable(rownr,6), mintable(rownr,7),
     &                mintable(rownr,3)-gamma1,
     &                mintable(rownr,3)-gamman
         write(6,201) location, l, m, mintable(rownr,3), 
     &                mintable(rownr,4), mintable(rownr,5), 
     &                mintable(rownr,6), mintable(rownr,7),
     &                mintable(rownr,3)-gamma1,
     &                mintable(rownr,3)-gamman
         gamman = mintable(rownr,3)
         do k = 1, 7, 1
            mintable(rownr,k) = mintable(location,k)
         enddo
         location = location + 1
         minimum  = mintable(location,3)
         rownr    = location
         goto 15
      endif
      
      
c**** search for the (l,m)-value associated with the lowest value of the 
c     discriminant (sort by alternation)

 1000 close(1)

      write(6,*)
      write(6,*) 
      write(6,*) '*********************'
      write(6,*) '* MISSION COMPLETED *'
      write(6,*) '*********************'
      write(6,*)
      write(6,*)
      write(6,*) output
      write(6,*)
      write(6,*)

      stop

      end



c================
c     subroutines
c================

c----------------------------------------------------------
      real*8 function discriminant(l,m,i,vp,vomega,sigma,
     &       aa,cc,dd,ee,ff,gg,rst,saa,scc,sdd,see,sff,sgg,srst)
c----------------------------------------------------------
c     Computation of the  discriminant. The discriminant compares the 
c     observed variations of the moments with theoretically calculated
c     expressions for these variations. The discriminant is based on all 
c     the amplitudes of the first three moments (given by aa, cc, dd, ff, 
c     gg, rst). Each term in the discriminant is  multiplied by a weight 
c     according to the amplitude that appears in that term. Therefore we 
c     make use of the uncertainties of the determined observational amplitudes.
c     {for further reading: C. Aerts, Astron. Astrophys.134, 115-122 (1996)}. 
       

      integer k, l, m, r
      real*8  i, vp, vomega, sigma, weight
      real*8  almk, bnmlk, hnlk, jnlk, nlm
      real*8  hhlkr, hh2lkr, hh3lkr, hh4lk, hh5lk
      real*8  vgl(7), f, Sf
      real*8  Kvalue, beta
      real*8   aa,  cc,  dd,  ee,  ff,  gg,  rst
      real*8  faa, fcc, fdd, fee, fff, fgg, frst
      real*8  saa, scc, sdd, see, sff, sgg, srst

      common/limbdarkening/beta
      common/alfa/Kvalue
      common/error/Sf
      

c     Calculation of each term of the discriminant

c**** term 1
c     Calculation of | |A(l,m,i)|-aa | (Aerts, C., 1993, PhDthesis, p.58)

      vgl(1) = abs(abs(2.d0*vp*nlm(l,m)/(1.d0+2.d0*beta/3.d0)*
     &         almk(l,m,0,i)*(jnlk(0,l,0)+Kvalue*hnlk(0,l,0)))-aa)

c**** term 2
c     Calculation of | |C(l,m,i)|-cc |^{1/2} (Aerts, C., 1993, PhDthesis, p.59)

      vgl(2)=0.0d0

      if (l.eq.0) then
         vgl(2) = (abs(vp*vp*(3.d0/20.d0)*(5.0d0+4.0d0*beta)/
     &             (3.d0+2.d0*beta)-cc))**(0.5d0)
      else
         do k = 1, l, 1
            vgl(2) = vgl(2) + 2.0d0 * ((-1.0d0)**k)*almk(l,m,k,i)*
     &           almk(l,m,-k,i)*(((1.0d0+Kvalue*(l+1.0d0))**(2.0d0))*
     &           (bnmlk(3,k,l,l)+beta*bnmlk(4,k,l,l))+
     &           ((Kvalue*(l+1.0d0))**(2.0d0))*(bnmlk(1,k,l+1,l+1)+
     &           beta*bnmlk(2,k,l+1,l+1))-2.0d0*Kvalue*(l+1.0d0)*
     &           (1.0d0+Kvalue*(l+1.0d0))*
     &           (bnmlk(2,k,l,l+1)+beta*bnmlk(3,k,l,l+1)))
         enddo
 
c     We only have taken the values k = 1, l into account, so we 
c     have to add a term with k = 0. 

         vgl(2) = vgl(2) + (almk(l,m,0,i)**(2.0d0))*
     &           (((1.0d0+Kvalue*(l+1.0d0))**(2.0d0))*
     &           (bnmlk(3,0,l,l)+beta*bnmlk(4,0,l,l))+
     &           ((Kvalue*(l+1.0d0))**(2.0d0))*(bnmlk(1,0,l+1,l+1)+
     &           beta*bnmlk(2,0,l+1,l+1))-2.0d0*Kvalue*(l+1.0d0)*
     &           (1.0d0+Kvalue*(l+1.0d0))*
     &           (bnmlk(2,0,l,l+1)+beta*bnmlk(3,0,l,l+1)))
         
         vgl(2)=(abs(abs(vgl(2)*vp*vp*(nlm(l,m)**(2.0d0))/
     &        (1.0d0+2.0d0*beta/3.0d0))-cc))**(0.5d0)
         
      endif
 

c**** term 3
c     Calculation of | |D(l,m,i)|-dd |^{1/2} (Aerts, C., 1993, PhDthesis, p.59)

      vgl(3) = 0.d0
      if (m.eq.0) then
  	 vgl(3) = dd**(0.5d0)
c        this expression  can easily be found by using the symmetry 
c        relations for the  a_{l,m,k}(i).

      else 
         vgl(3) = (abs(abs(2.d0*nlm(l,m)/(1.d0+2.d0*beta/3.d0)*vp*
     &            vomega*(almk(l,m,1,i)+almk(l,m,-1,i))*(jnlk(1,l,1)+
     &            Kvalue*hnlk(1,l,1)))-dd))**(0.5d0)
      endif  		  	  				

     
c**** term 4
c     Calculation of | |E(l,m,i)|-ee |^{1/2} (Aerts, C., 1993, PhDthesis, p.59)
     
      vgl(4) = 0.0d0

      do k = -l, l, 1
         vgl(4) = vgl(4) + (almk(l,m,k,i)**(2.0d0)) *
     &      ( (((1+Kvalue*(l+1.0d0))**(2.0d0)) *
     &         (bnmlk(3,k,l,l) + beta*bnmlk(4,k,l,l))) + 
     &        (((Kvalue*(l+1.0d0))**(2.0d0)) * 
     &         (bnmlk(1,k,l+1,l+1) + beta*bnmlk(2,k,l+1,l+1))) - 
     &        ((2.0d0*Kvalue*(l+1.0d0)*(1.0d0+Kvalue*(l+1.0d0)))*
     &         (bnmlk(2,k,l,l+1)+beta*bnmlk(3,k,l,l+1))) )
      enddo
      
      vgl(4) = ( abs( ee - 
     &                (abs(vgl(4)*vp*vp*(nlm(l,m)**(2.0d0)) / 
     &                              (1.0d0+(2.0d0*beta/3.0d0)) )) - 
     &                (((15.0d0+8.0d0*beta)/(60.0d0+40.0d0*beta))*
     &                                           vomega**(2.0d0)) - 
     &                (sigma**(2.0d0)) ) )**(0.5d0)

         
c**** term 5
c     Calculation of | |F(l,m,i)|-ff |^{1/3} (Aerts, C., 1993, PhDthesis, p.61)

      vgl(5) = 0.0d0

      if (l.eq.0) then
         vgl(5) = (abs((vp*vp*vp*(6.0d0+5.0d0*beta)/
     &               (60.0d0+40.0d0*beta))-ff))**(1.0d0/3.0d0)
      else
         do k = -l, l-1, 1
            do r = k+1, l, 1
               if (abs(k+r).le.l) then
                  vgl(5) = vgl(5) + (almk(l,m,k,i)*almk(l,m,r,i)*
     &                 ((-1.0d0)**(k+r+1))*almk(l,m,-k-r,i)*
     &                 hhlkr(l,k,r)*2.0d0)
               endif
            enddo
            if (abs(2*k).le.l) then
               vgl(5) = vgl(5) - (((almk(l,m,k,i))**(2.0d0))*
     &                  almk(l,m,-2*k,i)*hhlkr(l,k,k))
            endif
   
         enddo

         vgl(5) = (abs(abs(vgl(5)*vp*vp*vp*(nlm(l,m)**(3.0d0))/(2.0d0)/
     &    (1.0d0+2.0d0*beta/(3.0d0))) -ff))**(1.0d0/3.0d0)
      endif


c**** term 6
c     Calculation of | |G(l,m,i)|-gg |^{1/3} (Aerts, C., 1993, PhDthesis, p.61)

      vgl(6) = 0.d0
      if (m.eq.0) then
         vgl(6) = gg**(1.0d0/3.0d0)

c     this expression can easily be found by using the symmetry relations 
c        for the a_{l,m,k}(i) and the P_l^{m}.

      else
         do k = -l,l
            vgl(6) = vgl(6) + almk(l,m,k,i)*(almk(l,m,-1-k,i)*hh5lk(l,k)
     &               -almk(l,m,1-k,i)*hh4lk(l,k))
         enddo     
    
         vgl(6) = (abs(abs(vgl(6)*nlm(l,m)**2.d0*3.d0/(2.d0*
     &            (1.d0+2.d0*beta/3.d0))*vp**2.d0*vomega)-gg))
     &            **(1.d0/3.d0)
      
      endif


c**** term 7
c     Calculation of | |RST(l,m,i)|-rst |^{1/3} (Aerts, C., 1993, PhDthesis,
c     p.61)

      vgl(7) = 0.d0

c     Calculation of R(l,m,i) (Aerts, C., 1993, PhDthesis, p.61)

      do k = -l, l, 1
         do r = -l, l, 1
            if (abs(k+r) .le. l) then
               vgl(7) = vgl(7) + almk(l,m,k,i)*almk(l,m,r,i)
     &                  *almk(l,m,k+r,i)*hhlkr(l,k,r)
            endif
            if (abs(r-k) .le. l) then
               vgl(7) = vgl(7) + almk(l,m,k,i)*almk(l,m,r,i)
     &                  *almk(l,m,r-k,i)*hh2lkr(l,k,r)
               vgl(7) = vgl(7) + almk(l,m,k,i)*almk(l,m,r,i)
     &                  *almk(l,m,k-r,i)*hh3lkr(l,k,r)
            endif
         enddo
      enddo         
      vgl(7) = abs(vgl(7))*vp*vp*vp*(nlm(l,m)**3.d0)/(2.d0*
     &            (1.d0+2.d0*beta/3.d0))

c     Calculation of T(l,m,i) (Aerts, C., 1993, PhDthesis, p.61)

      vgl(7) = vgl(7)+abs(vp*nlm(l,m)*sigma**2.d0/(1.d0+2.d0*beta/3.d0)*
     &         6.d0*almk(l,m,0,i)*(jnlk(0,l,0)+Kvalue*hnlk(0,l,0)))

c     Calculation of S(l,m,i) (Aerts, C., 1993, PhDthesis, p.61)

      vgl(7) = vgl(7)+abs(vp*nlm(l,m)*vomega**2.d0/
     &         (1.d0+2.d0*beta/3.d0)*(3.d0*almk(l,m,0,i)*(jnlk(2,l,0)+
     &         Kvalue*hnlk(2,l,0))-(3.d0/2.d0)*(almk(l,m,2,i)+
     &         almk(l,m,-2,i))*(jnlk(2,l,2)+Kvalue*hnlk(2,l,2))))

      vgl(7) = (abs(abs(vgl(7))-rst))**(1.d0/3.d0)


c**** Calculation of the total weight.

      weight = aa/saa+cc/scc+dd/sdd+ee/see+ff/sff+gg/sgg+rst/srst

c**** We multiply each term of the discriminant by a dimensionless
c     weight according to the amplitude that appears in that term. 

      faa  =  aa/(saa*weight)
      fcc  =  cc/(scc*weight)
      fdd  =  dd/(sdd*weight)
      fee  =  ee/(see*weight)
      fff  =  ff/(sff*weight)
      fgg  =  gg/(sgg*weight)
      frst = rst/(srst*weight)

      vgl(1) = vgl(1)*faa
      vgl(2) = vgl(2)*fcc
      vgl(3) = vgl(3)*fdd
      vgl(4) = vgl(4)*fee
      vgl(5) = vgl(5)*fff
      vgl(6) = vgl(6)*fgg
      vgl(7) = vgl(7)*frst

c**** Calculation of the discriminant

      discriminant = dsqrt(vgl(1)**2.d0+vgl(2)**2.d0+vgl(3)**2.d0+
     &       +vgl(4)**2.d0+vgl(5)**2.d0+vgl(6)**2.d0+vgl(7)**2.d0)

c**** Calculation of the function f and the error Sf of this function. 
c     Sf is an indicator of the error of the discriminant induced 
c     by the error of the observed amplitudes.

      f = dsqrt((( (faa)**2.d0)*(aa**2.d0))+
     &          (( (fdd)**2.d0)*(cc))+
     &          (( (fdd)**2.d0)*(dd))+
     &          (( (fee)**2.d0)*(ee))+
     &          (( (fff)**2.d0)*( ff**(2.0d0/3.0d0)))+
     &          (( (fgg)**2.d0)*( gg**(2.0d0/3.0d0)))+
     &          (((frst)**2.d0)*(rst**(2.0d0/3.0d0))))
      
      Sf= dsqrt((   (faa**4.d0)* (saa**2.d0)*( aa**2.d0)  )+
     &          ((( (fcc**4.d0)* (scc**2.d0))+((fdd**4.d0)*(sdd**2.d0))+
     &            ( (fee**4.d0)* (see**2.d0)) ) / 4.0d0) +
     &          ((( (fff**4.d0)* (sff**2.d0)*( ff**(-2.0d0/3.0d0))) +
     &            ( (fgg**4.d0)* (sgg**2.d0)*( gg**(-2.0d0/3.0d0))) +
     &            ((frst**4.d0)*(srst**2.d0)*(rst**(-2.0d0/3.0d0))) ) /
     &                                                 9.0d0))/f


      return
      end
 

c----------------------------------
      real*8 function hh5lk(l,k)
c----------------------------------
c     Construction of L^{k,-1-k}_{l,l}

      parameter (nn=12)
      parameter (eps=3.d-14)
      integer j, k, l
      real*8  plm, int

      real*8  absc(nn), w(nn),  beta, Kvalue
      common/alfa/Kvalue
      common/integraal/absc,w
      common/limbdarkening/beta

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine  L^{k,-1-k}_{l,l}. The 
C     integral is given by the sum (j = 1 to nn) of w(j)*f(absc(j)), with f 
C     the integrand of  L^{k,-1-k}_{l,l}.

      int = 0.d0
      do j = 1, nn, 1
         int = int + w(j)*sqrt(1-absc(j)**2)*(absc(j)+beta*absc(j)**2)*
     &         (absc(j)*plm(l,k,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,k,absc(j))-plm(l+1,k,absc(j))))*
     &         (absc(j)*plm(l,-1-k,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,-1-k,absc(j))-plm(l+1,-1-k,absc(j))))
      enddo

      IF (abs(int).lt.eps) int = 0.0d0

      hh5lk = int
   
      return
      end


c----------------------------------
      real*8 function hh4lk(l,k)
c----------------------------------
c     Construction of L^{k,1-k}_{l,l}

      parameter (nn=12)
      parameter (eps=3.d-14)
      integer j, k, l
      real*8  plm, int

      real*8  absc(nn), w(nn),  beta, Kvalue
      common/alfa/Kvalue
      common/integraal/absc,w
      common/limbdarkening/beta

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine  L^{k,1-k}_{l,l}. The 
C     integral is given by the sum (j = 1 to nn) of w(j)*f(absc(j)), with f 
C     the integrand of  L^{k,1-k}_{l,l}.

      int = 0.d0
      do j = 1, nn, 1
         int = int + w(j)*sqrt(1-absc(j)**2)*(absc(j)+beta*absc(j)**2)*
     &         (absc(j)*plm(l,k,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,k,absc(j))-plm(l+1,k,absc(j))))*
     &         (absc(j)*plm(l,1-k,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,1-k,absc(j))-plm(l+1,1-k,absc(j))))
      enddo

      IF (abs(int).lt.eps) int = 0.0d0

      hh4lk = int
   
      return
      end


c----------------------------------
      real*8 function hh3lkr(l,k,r)
c----------------------------------
c     Construction of I^{k,r,k-r}_{l,l,l}

      parameter (nn=12)
      parameter (eps=3.d-14)
      integer j, k, l, r
      real*8  plm, int

      real*8  absc(nn), w(nn),  beta, Kvalue
      common/alfa/Kvalue
      common/integraal/absc,w
      common/limbdarkening/beta

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine I^{k,r,k-r}_{l,l,l}. The 
C     integral is given by the sum (j = 1 to nn) of w(j)*f(absc(j)), with f 
C     the integrand of I^{k,r,k-r}_{l,l,l}.

      int = 0.d0
      do j = 1, nn, 1
         int = int + w(j)*(absc(j)+beta*absc(j)**2)*
     &         (absc(j)*plm(l,k,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,k,absc(j))-plm(l+1,k,absc(j))))*
     &         (absc(j)*plm(l,r,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,r,absc(j))-plm(l+1,r,absc(j))))*
     &         (absc(j)*plm(l,k-r,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,k-r,absc(j))-plm(l+1,k-r,absc(j))))      
      enddo

      IF (abs(int).lt.eps) int = 0.0d0

      hh3lkr = int
   
      return
      end


c----------------------------------
      real*8 function hh2lkr(l,k,r)
c----------------------------------
c     Construction of I^{k,r,r-k}_{l,l,l}

      parameter (nn=12)
      parameter (eps=3.d-14)
      integer j, k, l, r
      real*8  plm, int

      real*8  absc(nn), w(nn),  beta, Kvalue
      common/alfa/Kvalue
      common/integraal/absc,w
      common/limbdarkening/beta

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine I^{k,r,r-k}_{l,l,l}. The 
C     integral is given by the sum (j = 1 to nn) of w(j)*f(absc(j)), with f 
C     the integrand of I^{k,r,r-k}_{l,l,l}.

      int = 0.d0
      do j = 1, nn, 1
         int = int + w(j)*(absc(j)+beta*absc(j)**2)*
     &         (absc(j)*plm(l,k,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,k,absc(j))-plm(l+1,k,absc(j))))*
     &         (absc(j)*plm(l,r,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,r,absc(j))-plm(l+1,r,absc(j))))*
     &         (absc(j)*plm(l,r-k,absc(j))+Kvalue*(l+1)*(absc(j)*
     &         plm(l,r-k,absc(j))-plm(l+1,r-k,absc(j))))      
      enddo

      IF (abs(int).lt.eps) int = 0.0d0

      hh2lkr = int
   
      return
      end


c---------------------------------
      real*8 function hhlkr(l,k,r)
c---------------------------------
c     Construction of I^{k,r,k+r}_{l,l,l}

      parameter (nn=12)
      parameter (eps=3.d-14)
      integer j, k, l, r
      real*8  plm, int

      real*8  absc(nn), w(nn),  beta, Kvalue
      common/alfa/Kvalue
      common/integraal/absc,w
      common/limbdarkening/beta

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine I^{k,r,k+r}_{l,l,l}. The 
C     integral is given by the sum (j = 1 to nn) of w(j)*f(absc(j)), with f 
C     the integrand of I^{k,r,k+r}_{l,l,l}.

      int = 0.d0
      do j = 1, nn, 1
         int = int + w(j)*(absc(j)+beta*absc(j)**2.d0)*
     &         (absc(j)*plm(l,k,absc(j))+Kvalue*(l+1.d0)*(absc(j)*
     &         plm(l,k,absc(j))-plm(l+1,k,absc(j))))*
     &         (absc(j)*plm(l,r,absc(j))+Kvalue*(l+1.d0)*(absc(j)*
     &         plm(l,r,absc(j))-plm(l+1,r,absc(j))))*
     &         (absc(j)*plm(l,k+r,absc(j))+Kvalue*(l+1.d0)*(absc(j)*
     &         plm(l,k+r,absc(j))-plm(l+1,k+r,absc(j))))      
      enddo

      IF (abs(int).lt.eps) int = 0.0d0

      hhlkr = int
   
      return
      end


c--------------------------------
      real*8 function hnlk(n,l,k) 
c--------------------------------
c     Construction of H^n_{l,k}

      parameter (nn=12)
      parameter (eps=3.d-14)
      integer j, k, l, n
      real*8  q, plm, int

      real*8  absc(nn), w(nn),  beta
      common/integraal/absc,w
      common/limbdarkening/beta

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine H^n_{l,k}. The integral is 
C     given by the sum (j = 1 to nn) of w(j)*f(absc(j)), with f the integrand 
C     of H^n_{l,k}.

      q = n
      int = 0.d0
      do j = 1, nn, 1
         int = int + w(j)*((absc(j)+beta*absc(j)**2.d0)*((1.d0-absc(j)
     &         **2.d0)**(q/2.d0))*(absc(j)*plm(l,k,absc(j))-
     &         plm(l+1,k,absc(j))))
      enddo

      IF (abs(int).lt.eps) int = 0.0d0

      hnlk = (l+1.d0)*int
   
      return
      end


c--------------------------------
      real*8 function jnlk(n,l,k) 
c--------------------------------
c     Construction of J^n_{l,k}

      parameter (nn=12)
      parameter (eps=3.d-14)
      integer j, k, l, n
      real*8  q, plm, int

      real*8  absc(nn), w(nn), beta
      common/integraal/absc,w
      common/limbdarkening/beta

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine J^n_{l,k}. The integral is 
C     given by the sum (j = 1 to nn) of w(j)*f(absc(j)), with f the integrand 
C     of J^n_{l,k}.

      q = n
      int = 0.d0
      do j = 1, nn, 1
        int = int + w(j)*((absc(j)**2.d0+beta*absc(j)**3.d0)*((1-absc(j)
     &         **2.d0)**(q/2.d0))*plm(l,k,absc(j)))       
      enddo

      IF (abs(int).lt.eps) int = 0.0d0

      jnlk = int
   
      return
      end


c-----------------------------------
      real*8 function bnmlk(n,m,l,k)
c----------------------------------- 
c     Construction of B^{n,m}_{l,k}
      
      parameter (abscissas=12)
      parameter (eps=3.d-14)
      integer j, k, l, m, n
      real*8  plm, int

      real*8  absc(abscissas), w(abscissas)
      common/integraal/absc,w

C     We will use  the computed abscissas and weights of the Gauss-Legendre
C     nn-point quadrature formula to determine B^{k,m}_{l,n}. The integral is 
C     given by the sum (i = 1 to nn) of w(i)*f(absc(i)), with f the integrand 
C     of B^{k,m}_{l,n}.


      int = 0.d0

      do j = 1, abscissas, 1
         int = int + w(j)*(absc(j)**n*plm(l,m,absc(j))*plm(k,m,absc(j)))        
      enddo
    
      IF (abs(int).lt.eps) int = 0.0d0
    
      bnmlk = int

      return
      end


c-------------------------------
      real*8 function plm(l,m,x)
c-------------------------------
c     Calculation of Legendre Polynomial P^{m}_{l}(x).
c     This function is based on the function proposed in the numerical 
c     recipes {nr}, but is adopted to conny's definition {conny} :
c     P^{m}_{l}(x)_{conny} = [(l-m)! / (-1)^m l!] P^{m}_{l}(x)_{nr}

      integer i, ll, l, m, lower
      real*8  pmm, pll, x, somx2, fact, pmmp1
      real*8  facul

      IF (abs(x).gt.1) THEN
         write(6,*)
         write(6,*) 'abs(x) = ',abs(x)
         write(6,*)
         pause 'bad argument'
      ENDIF

      IF (abs(m).gt.l) THEN
         plm = 0.0d0
         goto 492
      ENDIF

      lower = 0
      IF (m.lt.0) THEN
         m     = -m
         lower =  1
      ENDIF

c     P^{0}_{0} = 1
      pmm = 1.0d0
      plm = pmm

c     Calculation of P^{m}_{m} for m.gt.0
c     We use the formula : P^{m}_{m} = (-1)^m (2m-1)!! (1-x^2)^{m/2}
c     n!! means all odd number .le. n
      if (m.gt.0) then 
         somx2 = sqrt((1.0d0-x)*(1.0d0+x))
         fact=1.0d0
         do i = 1, m
            pmm  = -pmm*fact*somx2
            fact = fact+2.0d0
         enddo
      endif

      if (l.eq.m) then
c     *** Adapt the solution to conny's definition ***
         plm = pmm / ( ((-1.0d0)**m) * facul(m) )
      else
c        Calculation of P^{m}_{m+1}
c        We use the formula : P^{m}_{m+1} = x (2m+1) P^{m}_{m}
         pmmp1 = x*(2.0d0*M+1)*pmm
         if (l.eq.m+1) then
c     *** Adapt the solution to conny's definition ***
            plm = pmmp1 / ( ((-1.0d0)**m) * facul(m+1) )
         else
c           Calculation of P^{m}_{l} for l.gt.(m+1)
c           We use the formula : 
c           (l-m) P^{m}_{l} = x (2l-1) P^{m}_{l-1} - (l+m-1) P^{m}_{l-2}
            do ll = m+2, l
               pll   = (x*(2.0d0*ll-1)*pmmp1-(ll+m-1.0d0)*pmm)/(ll-m)
               pmm   = pmmp1
               pmmp1 = pll
            enddo
c     *** Adapt the solution to conny's definition ***
            plm = pll / ( ((-1.0d0)**m) * facul(l) ) * facul(l-m)
         endif
      endif

      IF (lower.eq.1) then
         plm = (-1.0d0)**m * plm
         m = -m
      ENDIF

 492  return
      end
     

c----------------------------------
      real*8 function almk(l,m,k,i)
c----------------------------------
c     Definition of a_{l,m,k}(i)
     
      integer j, k, l, m, ul,ll
      real*8  facul, i, res

c     Upperlimit ul = min(l-k,l-m)
      ul = min(l-k,l-m)

c     Lowerlimit ll = max(0,-k-m)
      ll = max(0,-k-m)

      res = 0.0d0

      do j = ll, ul, 1
         res = res + (  ( (-1.0d0)**(j+k+l) ) * 
     &    ((dcos(i/2.0d0)**(2*j+m+k))*(dsin(i/2.0d0)**(2*l-2*j-m-k)))/ 
     &    (facul(j)*facul(l-m-j)*facul(l-k-j)*facul(m+k+j)) )
      enddo 

      almk = res*facul(l-m)*facul(l+m)

      return
      end
      

c---------------------------
      real*8 function nlm(l,m)
c---------------------------
c     Definition of N^{m}_{l}

      integer  l, m
      real*8   facul
     
      nlm = facul(l)*dsqrt((2*l+1)/(facul(l-m)*facul(l+m)))

      return
      end


c----------------------
      subroutine gauleg
c----------------------
c     Given the lower and upper limits of integration x1 and x2, this 
c     function computes arrays absc and w of length abscissas, containing the 
c     abscissas and weights of the Gauss-Legendre nn-point quadrature 
c     formula. This function is based upon the function proposed in the 
c     numerical recipes {nr}.

      parameter (x1=0.0d0,x2=1.0d0)
      parameter (eps=3.d-14)
      parameter (abscissas=12)
c     implicit real*8 (a-h,o-z)
      real*8   xm, xl

      real*8  absc(abscissas), w(abscissas)
      common/integraal/absc,w

c     The roots are symmetric in the interval, so we only have to find 
c     half of them.

      q = (abscissas+1.0)/2.0d0
      xm = 0.5d0*(x2+x1)    
      xl = 0.5d0*(x2-x1)

c     Loop over the desired roots

      do 12 i = 1, aint(q), 1
         z = cos(3.141592654d0*(i-.25d0)/(abscissas+.5d0))

 1       continue
            p1 = 1.d0
            p2 = 0.d0

c           Loop up the recurrence relation to get the Legendre polynomial
c           evaluated at z.            
            do 11 j =  1, abscissas, 1
               p3 = p2
               p2 = p1
               p1 = ((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
 11         continue
c           p2 is now the desired Legendre polynomial. We next compute pp, its
c           derivative, by a standard relation involving also p2, the polynomial
c           of one lower order.   
  
            pp = abscissas*(z*p1-p2)/(z*z-1.d0)
            z1 = z
            z = z1-p1/pp
            
         if (abs(z-z1) .GT. eps) go to 1
        
c        We scale the root to the desired interval and we put in its 
c        symmetric counterpart. We also compute the weight and its 
c        symmetric counterpart. 

         absc(i) = xm-xl*z
         absc(abscissas+1-i) = xm+xl*z
         W(i) = 2.d0*xl/((1.d0-z*z)*pp*pp)
         W(abscissas+1-i) = W(i)    
 
 12   continue
      
      return
      end       
                  

c-----------------------------
      REAL*8 function facul(n)                                            
c-----------------------------                                                 
c     This function calculates n!

      integer ihulp,n
      real*8  res

      res = 1.0d0
      if (n.le.1) GOTO 10  

      do ihulp = 2, n                                                           
         res = res*ihulp
      enddo

 10   facul = res

      return                                                                   
      end                                                                    

c-----------------------------------------------------------------------
